@@ -22,19 +22,19 @@
 #include "av-meta-reader.h"
 
 static void
-av_meta_reader_init (AVMetaReader *reader)
+av_meta_reader_init (AvMetaReader *reader)
 {
 }
 
 static void
-av_meta_reader_class_init (AVMetaReaderClass *klass)
+av_meta_reader_class_init (AvMetaReaderClass *klass)
 {
 }
 
-G_DEFINE_TYPE (AVMetaReader, av_meta_reader, G_TYPE_OBJECT)
+G_DEFINE_TYPE (AvMetaReader, av_meta_reader, G_TYPE_OBJECT)
 
 gboolean
-av_meta_reader_read (AVMetaReader *reader,
+av_meta_reader_read (AvMetaReader *reader,
                      DmapAvRecord *record,
                      const gchar *path,
                      GError **error)
@@ -43,7 +43,7 @@ av_meta_reader_read (AVMetaReader *reader,
 }
 
 GOptionGroup *
-av_meta_reader_get_option_group (AVMetaReader *reader)
+av_meta_reader_get_option_group (AvMetaReader *reader)
 {
 	return AV_META_READER_GET_CLASS (reader)->get_option_group (reader);
 }
