@@ -22,9 +22,12 @@
 #include "db-builder-gdir.h"
 
 #include <libdmapsharing/dmap.h>
+#include <config.h>
+#include <stdbool.h>
 
 struct DbBuilderGDirPrivate {
 	GSList          *monitor_list;
+	GHashTable      *creates_underway;
 	DmapDb          *db;
 	DmapContainerDb *container_db; // NULL if we don't want directory containers.
 };
@@ -87,7 +90,78 @@ _build_db_starting_at(DbBuilder *builder,
                       GError **error);
 
 static void
-_monitor_directory (DbBuilderGDir *builder, const char *path);
+_monitor_directory (DbBuilderGDir *builder, const char *path, GCallback cb, void *user_data);
+
+static gboolean
+_handle_path(DbBuilderGDir *builder,
+             const char *path,
+             DmapContainerRecord *container_record,
+             GError **error);
+
+static void
+_file_changed_cb (GFileMonitor *monitor,
+                  GFile *file,
+                  GFile *other_file,
+                  GFileMonitorEvent event_type,
+                  DbBuilderGDir *builder)
+{
+	g_assert (G_IS_FILE_MONITOR (monitor));
+	g_assert (G_IS_FILE (file));
+	g_assert (NULL != builder);
+
+	// FIXME: Need to lock database?
+
+	gboolean ok;
+	GError *error = NULL;
+
+	gchar *path = g_file_get_path (file);
+	if (NULL == path) {
+		g_warning ("Could not determine path for changed file");
+		goto done;
+	}
+
+	switch (event_type) {
+		case G_FILE_MONITOR_EVENT_CREATED:
+			g_debug ("New file start: %s", path);
+			g_hash_table_add(builder->priv->creates_underway, g_strdup(path));
+			break;
+		case G_FILE_MONITOR_EVENT_CHANGED:
+			g_debug ("New file change: %s", path);
+			break;
+		case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+			g_debug ("New file done: %s", path);
+			ok = g_hash_table_contains(builder->priv->creates_underway, path);
+			if (!ok) {
+				g_warning("Done hint, but no created %s event", path);
+				goto done;
+			}
+
+			g_hash_table_remove(builder->priv->creates_underway, path);
+
+			ok = _handle_path (builder, path, NULL, &error);
+			if (!ok) {
+				g_warning("Error handling new file: %s", error->message);
+				goto done;
+			}
+
+			break;
+		case G_FILE_MONITOR_EVENT_DELETED:
+			// FIXME: Not implemented, need remove function in database.
+			g_debug ("Removed file: %s", path);
+			g_warning ("Removing file %s not implemented", path);
+			break;
+		default:
+			g_warning ("Unhandled change to %s: %d", path, event_type);
+			break;
+	}
+
+done:
+	if (NULL != path) {
+		g_free (path);
+	}
+
+	return;
+}
 
 static gboolean
 _handle_path(DbBuilderGDir *builder,
@@ -120,7 +194,7 @@ _handle_path(DbBuilderGDir *builder,
 		}
 		g_object_unref (record);
 
-		_monitor_directory (DB_BUILDER_GDIR (builder), path);
+		_monitor_directory (DB_BUILDER_GDIR (builder), path, G_CALLBACK(_file_changed_cb), builder);
 	} else {
 		gchar *location;
 		guint id = 0;
@@ -154,60 +228,12 @@ done:
 }
 
 static void
-_file_changed_cb (GFileMonitor *monitor,
-                  GFile *file,
-                  GFile *other_file,
-                  GFileMonitorEvent event_type,
-                  DbBuilderGDir *builder)
+_monitor_directory (DbBuilderGDir *builder, const char *path, GCallback cb, void *user_data)
 {
-	g_assert (G_IS_FILE_MONITOR (monitor));
-	g_assert (G_IS_FILE (file));
-	g_assert (NULL != builder);
-
-	// FIXME: Need to lock database?
-
-	gboolean ok;
-	GError *error = NULL;
-
-	gchar *path = g_file_get_path (file);
-	if (NULL == path) {
-		g_warning ("Could not determine path for changed file");
-		goto done;
+	if (NULL != builder) {
+		g_assert (IS_DB_BUILDER_GDIR (builder));
 	}
 
-	switch (event_type) {
-		case G_FILE_MONITOR_EVENT_CREATED:
-			g_debug ("New file: %s", path);
-
-			ok = _handle_path (builder, path, NULL, &error);
-			if (!ok) {
-				g_warning("Error handling new file: %s", error->message);
-				goto done;
-			}
-
-			break;
-		case G_FILE_MONITOR_EVENT_DELETED:
-			// FIXME: Not implemented, need remove function in database.
-			g_debug ("Removed file: %s", path);
-			g_warning ("Removing file %s not implemented", path);
-			break;
-		default:
-			g_warning ("Unhandled change to %s: %d", path, event_type);
-			break;
-	}
-
-done:
-	if (NULL != path) {
-		g_free (path);
-	}
-
-	return;
-}
-
-static void
-_monitor_directory (DbBuilderGDir *builder, const char *path)
-{
-	g_assert (IS_DB_BUILDER_GDIR (builder));
 	g_assert (NULL != path);
 
 	GFile *dir = g_file_new_for_path (path);
@@ -223,13 +249,13 @@ _monitor_directory (DbBuilderGDir *builder, const char *path)
 		goto done;
         }
 
-	gchar *file = g_file_get_basename (dir);
-	g_debug ("Monitoring %s", file);
-	g_free (file);
+	g_debug ("Monitoring %s", path);
 
-        g_signal_connect (monitor, "changed", (GCallback) _file_changed_cb, builder);	
+        g_signal_connect (monitor, "changed", cb, user_data);
 
-        builder->priv->monitor_list = g_slist_append (builder->priv->monitor_list, monitor);
+	if (NULL != builder) {
+		builder->priv->monitor_list = g_slist_append (builder->priv->monitor_list, monitor);
+	}
 
 done:
 	if (NULL != dir) {
@@ -239,6 +265,90 @@ done:
 	return;
 }
 
+#ifdef HAVE_CHECK
+
+typedef struct {
+	GMainLoop *loop;
+	gint state;
+	GFile *file;
+} MonitorData;
+
+static void
+_test_file_changed_cb (GFileMonitor *monitor,
+                       GFile *file,
+                       GFile *other_file,
+                       GFileMonitorEvent event_type,
+                       MonitorData *data)
+{
+	data->state++;
+}
+
+static gboolean
+_quit_idle(gpointer _data)
+{
+	MonitorData *data = _data;
+
+	g_main_loop_quit(data->loop);
+
+	return G_SOURCE_REMOVE;
+}
+
+static gboolean
+_delete_file_idle(gpointer _data)
+{
+	MonitorData *data = _data;
+	GError *error = NULL;
+
+	g_file_delete(data->file, NULL, &error);
+	g_assert_no_error (error);
+
+	g_idle_add(_quit_idle, data);
+
+	return G_SOURCE_REMOVE;
+}
+
+static gboolean
+_create_file_idle(gpointer _data)
+{
+	MonitorData *data = _data;
+	GError *error = NULL;
+
+	g_file_create(data->file, 0, NULL, &error);
+	g_assert_no_error (error);
+
+	g_idle_add(_delete_file_idle, data);
+
+	return G_SOURCE_REMOVE;
+}
+
+START_TEST(test_monitor_directory)
+{
+	gchar *tmpdir;
+	GError *error = NULL;
+	GFile *dir;
+	MonitorData data;
+
+	tmpdir = g_dir_make_tmp("db-test-builder-gdir_XXXXXX", &error);
+	g_assert_no_error (error);
+
+	dir = g_file_new_for_path(tmpdir);
+
+	data.file  = g_file_get_child(dir, "test-file");
+	data.loop  = g_main_loop_new(NULL, FALSE);
+	data.state = 0;
+
+	_monitor_directory(NULL, tmpdir, G_CALLBACK(_test_file_changed_cb), &data);
+
+	g_idle_add(_create_file_idle, &data);
+
+	g_main_loop_run(data.loop);
+
+	fail_unless(data.state = 1);
+}
+END_TEST
+
+#endif
+
 static gboolean
 _build_db_starting_at(DbBuilder *builder,
                       const char *dir,
@@ -276,7 +386,7 @@ _build_db_starting_at(DbBuilder *builder,
 		}
 	}
 
-	_monitor_directory (DB_BUILDER_GDIR (builder), dir);
+	_monitor_directory (DB_BUILDER_GDIR (builder), dir, G_CALLBACK(_file_changed_cb), builder);
 
 done:
 	if (NULL != d) {
@@ -291,9 +401,10 @@ db_builder_gdir_init (DbBuilderGDir *builder)
 {
         builder->priv = DB_BUILDER_GDIR_GET_PRIVATE (builder);
 
-	builder->priv->monitor_list = NULL;
-	builder->priv->db           = NULL;
-	builder->priv->container_db = NULL;
+	builder->priv->monitor_list     = NULL;
+	builder->priv->creates_underway = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
+	builder->priv->db               = NULL;
+	builder->priv->container_db     = NULL;
 }
 
 static void
@@ -311,6 +422,8 @@ _finalize (GObject *object)
 	if (NULL != builder->priv->monitor_list) {
 		g_slist_free_full (builder->priv->monitor_list, g_object_unref);
 	}
+
+	g_hash_table_destroy(builder->priv->creates_underway);
 }
 
 static void db_builder_gdir_class_init (DbBuilderGDirClass *klass)
@@ -360,3 +473,18 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (DbBuilderGDir,
                                 TYPE_DB_BUILDER,
                                 0,
                                 G_ADD_PRIVATE_DYNAMIC (DbBuilderGDir))
+
+#ifdef HAVE_CHECK
+
+Suite *db_test_builder_gdir_suite(void)
+{
+	Suite *s = suite_create("db-test-builder-gdir");
+
+	TCase *tc_monitor_directory = tcase_create("test_monitor_directory");
+	tcase_add_test(tc_monitor_directory, test_monitor_directory);
+	suite_add_tcase(s, tc_monitor_directory);
+
+	return s;
+}
+
+#endif
