@@ -405,16 +405,33 @@ _serve (protocol_id_t protocol,
         GError **error)
 {
 	DmapDb *db = NULL;
+	int rc;
 	gboolean ok;
 	DmapShare *share = NULL;
 	DmapContainerDb *container_db = NULL;
 
-	gchar *db_protocol_dir = g_strconcat (_db_dir ? _db_dir : DEFAULT_DBDIR,
-	                                     "/",
-	                                      _protocol_map[protocol],
-	                                      NULL);
 	g_assert (_db_module);
 
+	gchar *db_protocol_dir = g_build_path (G_DIR_SEPARATOR_S,
+	                                       _db_dir ? _db_dir : DEFAULT_DBDIR,
+	                                       share_def->name,
+	                                       _protocol_map[protocol],
+	                                       NULL);
+
+	ok = g_file_test (db_protocol_dir, G_FILE_TEST_EXISTS);
+	if (!ok) {
+		rc = g_mkdir_with_parents (db_protocol_dir, 0700);
+		if (-1 == rc) {
+			g_set_error (error,
+			             DMAP_ERROR,
+			             DMAP_STATUS_FAILED,
+			             "Failed to create %s: %s",
+			             db_protocol_dir,
+			             g_strerror (errno));
+			goto done;
+		}
+	}
+
 	db = DMAP_DB (util_object_from_module (TYPE_DMAPD_DMAP_DB, 
 	                                  _module_dir,
 					  _db_module,
@@ -479,8 +496,13 @@ _serve (protocol_id_t protocol,
 	}
 
 done:
-	g_object_unref (container_db);
-	g_object_unref (db);
+	if (NULL != container_db) {
+		g_object_unref (container_db);
+	}
+
+	if (NULL != db) {
+		g_object_unref (db);
+	}
 
 	g_free (db_protocol_dir);
 
