@@ -193,7 +193,7 @@ util_gst_transcode_cache (guint id, DmapAvRecord *record, db_dir_and_target_tran
 
 	format2 = dmap_utils_mime_to_format (df->target_transcode_mimetype);
 	if (NULL == format2) {
-		g_warning ("Cannot transcode %s\n", df->target_transcode_mimetype);
+		g_warning ("Cannot transcode %s", df->target_transcode_mimetype);
 		goto _return;
 	}
 
@@ -225,7 +225,7 @@ util_gst_transcode_cache (guint id, DmapAvRecord *record, db_dir_and_target_tran
 	/* Replace previous location with URI to transcoded file. */
 	cacheuri = g_filename_to_uri(cachepath, NULL, NULL);
 	if (NULL == cacheuri) {
-		g_warning ("Could not convert %s to URI.\n", cachepath);
+		g_warning ("Could not convert %s to URI", cachepath);
 		goto _return;
 	}
 
